#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in /LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: David C. Morrill
# Description: <Traits component>
#------------------------------------------------------------------------------



from traits.testing.unittest_tools import unittest

from ..api import HasTraits, Category, Str


class Base(HasTraits):
    y = Str("Base y")
    z = Str("Base z")


class BaseExtra(Category, Base):
    x = Str("BaseExtra x")


class BasePlus(Category, Base):
    p = Str("BasePlus p")
#   z = Str("BasePlus z")    overrides not allowed.


class BasePlusPlus(BasePlus):
    pp = Str("BasePlusPlus pp")


class CategoryTestCase(unittest.TestCase):
    """ Test cases for traits category """

    def setUp(self):
        self.base = Base()
        return

    def test_base_category(self):
        """ Base class with traits """
        self.assertEqual(self.base.y, "Base y", msg="y != 'Base y'")
        self.assertEqual(self.base.z, "Base z", msg="z != 'Base z'")
        return

    def test_extra_extension_category(self):
        """ Base class extended with a category subclass """
        self.assertEqual(self.base.x, "BaseExtra x", msg="x != 'BaseExtra x'")
        return

    def test_plus_extension_category(self):
        """ Base class extended with two category subclasses """
        self.assertEqual(self.base.x, "BaseExtra x", msg="x != 'BaseExtra x'")
        self.assertEqual(self.base.p, "BasePlus p", msg="p != 'BasePlus p'")
        return

    def test_subclass_extension_category(self):
        """ Category subclass does not extend base class.
        This test demonstrates that traits allows subclassing of a category
        class, but that the traits from the subclass are not actually added
        to the base class of the Category.
        Seems like the declaration of the subclass (BasePlusPlus) should fail.
        """
        try:
            x = self.base.pp
            self.fail(msg="base.pp should have thrown AttributeError "
                          "as Category subclassing is not supported.")
        except AttributeError:
            pass

        basepp = BasePlusPlus()
        return

    def test_subclass_instance_category(self):
        """ Category subclass instantiation not supported.
        This test demonstrates that traits allows subclassing of a category
        class, that subclass can be instantiated, but the traits of the parent
        class are not inherited.
        Seems like the declaration of the subclass (BasePlusPlus) should fail.
        """
        bpp = BasePlusPlus()
        self.assertEqual(bpp.pp, "BasePlusPlus pp",
                         msg="pp != 'BasePlusPlus pp'")

        try:
            self.assertEqual(bpp.p, "BasePlus p", msg="p != 'BasePlus p'")
            self.fail(msg="bpp.p should have thrown SystemError as "
                          "instantiating a subclass of a category is not "
                          "supported.")
        except SystemError:
            pass
        return

#
# support running this test individually, from the command-line as a script
#
if __name__ == '__main__':
    unittest.main()

#### EOF ######################################################################



from traits.testing.unittest_tools import unittest

from traits.api import Enum, HasTraits, List, Property, TraitError


class ExampleModel(HasTraits):
    valid_models = Property(List)
    root = Enum(values='valid_models')

    def _get_valid_models(self):
        return ['model1', 'model2', 'model3']


class EnumTestCase(unittest.TestCase):
    def test_valid_enum(self):
        example_model = ExampleModel(root='model1')
        example_model.root = 'model2'

    def test_invalid_enum(self):
        example_model = ExampleModel(root='model1')

        def assign_invalid():
            example_model.root = 'not_valid_model'

        self.assertRaises(TraitError, assign_invalid)



from traits.testing.unittest_tools import unittest

from traits.api import Any, HasStrictTraits, Str


class TestSpecialEvent(unittest.TestCase):
    """ Test demonstrating special change events using the 'event' metadata.
    """

    def setUp(self):
        self.change_events = []
        self.foo = Foo(test=self)

    def test_events(self):
        self.foo.val = 'CHANGE'

        values = ['CHANGE']
        self.assertEqual(self.change_events, values)

    def test_instance_events(self):
        foo = self.foo
        foo.add_trait('val2', Str(event='the_trait'))
        foo.val2 = 'CHANGE2'

        values = ['CHANGE2']
        self.assertEqual(self.change_events, values)


class Foo(HasStrictTraits):
    val = Str(event='the_trait')
    test = Any(None)

    def _the_trait_changed(self, new):
        if self.test is not None:
            self.test.change_events.append(new)



# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in /LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: David C. Morrill
# Date: 03/03/2003
# Description: Perform timing tests on various trait styles to determine the
#              amount of overhead that traits add.



from time import time
from ..api import Any, DelegatesTo, HasTraits, Int, Range

# Number of iterations to perform:
n = 1000000

# Loop overhead time (actual value determined first time a measurement is made)
t0 = -1.0


#  Measure how long it takes to execute a specified function:
def measure(func):
    now = time()
    func()
    return time() - now


#  'Old style' Python attribute get/set:
class old_style_value:

    def measure(self, reference_get=1.0, reference_set=1.0):
        global t0
        self.init()

        if t0 < 0.0:
            t0 = measure(self.null)
        t1 = measure(self.do_get)
        t2 = measure(self.do_set)

        scale = 1.0e6 / n
        get_time = max(t1 - t0, 0.0) * scale
        set_time = max(t2 - t0, 0.0) * scale

        return get_time, set_time

    def null(self):
        for i in range(n):
            pass

    def init(self):
        self.value = -1

    def do_set(self):
        for i in range(n):
            self.value = i

    def do_get(self):
        for i in range(n):
            self.value


#  'New style' Python attribute get/set:
class new_style_value(object):

    def measure(self):
        global t0
        self.init()

        if t0 < 0.0:
            t0 = measure(self.null)
        t1 = measure(self.do_get)
        t2 = measure(self.do_set)

        scale = 1.0e6 / n
        get_time = max(t1 - t0, 0.0) * scale
        set_time = max(t2 - t0, 0.0) * scale

        return get_time, set_time

    def null(self):
        for i in range(n):
            pass

    def init(self):
        self.value = -1

    def do_set(self):
        for i in range(n):
            self.value = i

    def do_get(self):
        for i in range(n):
            self.value


#  Python 'property' get/set:
class property_value(new_style_value):

    def get_value(self):
        return self._value

    def set_value(self, value):
        self._value = value

    value = property(get_value, set_value)


#  Python 'global' get/set:
class global_value(new_style_value):

    def init(self):
        global gvalue
        gvalue = -1

    def do_set(self):
        global gvalue
        for i in range(n):
            gvalue = i

    def do_get(self):
        global gvalue
        for i in range(n):
            gvalue


#  Trait that can have any value:
class any_value(HasTraits, new_style_value):

    value = Any


#  Trait that can only have 'float' values:
class int_value(any_value):

    value = Int


#  Trait that can only have 'range' values:
class range_value(any_value):

    value = Range(-1, 2000000000)


#  Executes method when float trait is changed:
class change_value(int_value):

    def _value_changed(self, old, new):
        pass


#  Notifies handler when float trait is changed:
class monitor_value(int_value):

    def init(self):
        self.on_trait_change(self.on_value_change, 'value')

    def on_value_change(self, object, trait_name, old, new):
        pass


#  Float trait is delegated to another object:
class delegate_value(HasTraits, new_style_value):

    value = DelegatesTo('delegate')
    delegate = Any

    def init(self):
        self.delegate = int_value()


#  Float trait is delegated through one object to another object:
class delegate_2_value(delegate_value):

    def init(self):
        delegate = delegate_value()
        delegate.init()
        self.delegate = delegate


#  Float trait is delegated through two objects to another object:
class delegate_3_value(delegate_value):

    def init(self):
        delegate = delegate_2_value()
        delegate.init()
        self.delegate = delegate


#  Run the timing measurements:
def report(name, get_time, set_time, ref_get_time, ref_set_time):
    """ Return string containing a benchmark report.

    The arguments are the name of the benchmark case, the times to do a 'get'
    or a 'set' operation for that benchmark case in usec, and the
    corresponding times for a reference operation (e.g., getting and
    setting an attribute on a new-style instance.
    """

    template = (
        '{name:^30}: Get {get_time:02.3f} us (x {get_speed_up:02.3f}), '
        'Set {set_time:02.3f} us (x {set_speed_up:02.3f})'
        )

    report = template.format(
        name=name,
        get_time=get_time,
        get_speed_up=ref_get_time / get_time,
        set_time=set_time,
        set_speed_up=ref_set_time / set_time,
        )

    return report


def run_benchmark(klass, ref_get_time, ref_set_time):
    benchmark_name = klass.__name__
    get_time, set_time = klass().measure()
    print(report(benchmark_name,
                 get_time, set_time,
                 ref_get_time, ref_set_time))


def main():
    ref_get_time, ref_set_time = new_style_value().measure()

    benchmarks = [
        global_value,
        old_style_value,
        new_style_value,
        property_value,
        any_value,
        int_value,
        range_value,
        change_value,
        monitor_value,
        delegate_value,
        delegate_2_value,
        delegate_3_value
    ]

    for benchmark in benchmarks:
        run_benchmark(benchmark, ref_get_time, ref_set_time)

if __name__ == '__main__':
    main()



#  Copyright (c) 2007, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in /LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt



from traits.testing.unittest_tools import unittest

from ..api import HasTraits, Int, Range, Long, TraitError


class A(HasTraits):
    i = Int
    l = Long
    r = Range(2, 9223372036854775807)


class TraitIntRangeLong(unittest.TestCase):
    def test_int(self):
        "Test to make sure it is legal to set an Int trait to a long value"
        a = A()
        a.i = 1
        a.i = 10

    def test_long(self):
        "Test if it is legal to set a Long trait to an int value"
        a = A()
        a.l = 10
        a.l = 100

    def test_range(self):
        "Test a range trait with longs being set to an int value"
        a = A()
        a.r = 256
        a.r = 20
        self.assertRaises(TraitError, a.trait_set, r=1)
        self.assertRaises(TraitError, a.trait_set, r=9223372036854775808)

if __name__ == '__main__':
    unittest.main()



#-----------------------------------------------------------------------------
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought util package component>
#
#-----------------------------------------------------------------------------

""" A simple topological sort on a dictionary graph.
"""

class CyclicGraph(Exception):
    """
    Exception for cyclic graphs.
    """
    def __init__(self):
        Exception.__init__(self, "Graph is cyclic")


def topological_sort(graph):
    """
    Returns the nodes in the graph in topological order.
    """
    discovered = {}
    explored = {}
    order = []
    def explore(node):
        children = graph.get(node, [])
        for child in children:
            if child in explored:
                pass
            elif child in discovered:
                raise CyclicGraph()
            else:
                discovered[child] = 1
                explore(child)
        explored[node] = 1
        order.append(node)

    for node in list(graph.keys()):
        if node not in explored:
            explore(node)
    order.reverse()
    return order





""" Test the ABC functionality.
"""
import abc
import warnings

from traits.testing.unittest_tools import unittest

from ..api import ABCHasTraits, ABCMetaHasTraits, HasTraits, Int, Float


class TestNew(unittest.TestCase):
    """ Test that __new__ works correctly.
    """

    def setUp(self):
        self.old_filters = warnings.filters[:]
        warnings.simplefilter('error', DeprecationWarning)

    def tearDown(self):
        warnings.filters[:] = self.old_filters

    def test_new(self):
        # Should not raise DeprecationWarning.
        HasTraits(x=10)


class AbstractFoo(ABCHasTraits):
    x = Int(10)
    y = Float(20.0)

    @abc.abstractmethod
    def foo(self):
        raise NotImplementedError()

    @abc.abstractproperty
    def bar(self):
        raise NotImplementedError()


class ConcreteFoo(AbstractFoo):
    def foo(self):
        return 'foo'

    @property
    def bar(self):
        return 'bar'


class FooLike(HasTraits):
    x = Int(10)
    y = Float(20.0)

    def foo(self):
        return 'foo'

    @property
    def bar(self):
        return 'bar'


AbstractFoo.register(FooLike)


class AbstractBar(object, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def bar(self):
        raise NotImplementedError()


class TestABC(unittest.TestCase):
    def test_basic_abc(self):
        self.assertRaises(TypeError, AbstractFoo)
        concrete = ConcreteFoo()
        self.assertEqual(concrete.foo(), 'foo')
        self.assertEqual(concrete.bar, 'bar')
        self.assertEqual(concrete.x, 10)
        self.assertEqual(concrete.y, 20.0)
        self.assertTrue(isinstance(concrete, AbstractFoo))

    def test_registered(self):
        foolike = FooLike()
        self.assertTrue(isinstance(foolike, AbstractFoo))

    def test_post_hoc_mixing(self):
        class TraitedBar(HasTraits, AbstractBar, metaclass=ABCMetaHasTraits):
            x = Int(10)

            def bar(self):
                return 'bar'

        traited = TraitedBar()
        self.assertTrue(isinstance(traited, AbstractBar))
        self.assertEqual(traited.x, 10)



""" Tests for the dynamic notifiers. """
import gc

from traits import _py2to3

from traits.api import Event, Float, HasTraits, List, on_trait_change
from traits.testing.unittest_tools import unittest

from traits import trait_notifiers


class DynamicNotifiers(HasTraits):

    ok = Float
    fail = Float
    priority_test = Event

    # Lists where we accumulate the arguments of calls to the traits notifiers.
    rebind_calls_0 = List
    rebind_calls_1 = List
    rebind_calls_2 = List
    rebind_calls_3 = List
    rebind_calls_4 = List
    exceptions_from = List
    prioritized_notifications = List

    #### 'ok' trait listeners

    @on_trait_change('ok')
    def method_listener_0(self):
        self.rebind_calls_0.append(True)

    @on_trait_change('ok')
    def method_listener_1(self, new):
        self.rebind_calls_1.append(new)

    @on_trait_change('ok')
    def method_listener_2(self, name, new):
        self.rebind_calls_2.append((name, new))

    @on_trait_change('ok')
    def method_listener_3(self, obj, name, new):
        self.rebind_calls_3.append((obj, name, new))

    @on_trait_change('ok')
    def method_listener_4(self, obj, name, old, new):
        self.rebind_calls_4.append((obj, name, old, new))

    #### 'fail' trait listeners

    @on_trait_change('fail')
    def failing_method_listener_0(self):
        self.exceptions_from.append(0)
        raise Exception('error')

    @on_trait_change('fail')
    def failing_method_listener_1(self, new):
        self.exceptions_from.append(1)
        raise Exception('error')

    @on_trait_change('fail')
    def failing_method_listener_2(self, name, new):
        self.exceptions_from.append(2)
        raise Exception('error')

    @on_trait_change('fail')
    def failing_method_listener_3(self, obj, name, new):
        self.exceptions_from.append(3)
        raise Exception('error')

    @on_trait_change('fail')
    def failing_method_listener_4(self, obj, name, old, new):
        self.exceptions_from.append(4)
        raise Exception('error')

    def low_priority_first(self):
        self.prioritized_notifications.append(0)

    def high_priority_first(self):
        self.prioritized_notifications.append(1)

    def low_priority_second(self):
        self.prioritized_notifications.append(2)

    def high_priority_second(self):
        self.prioritized_notifications.append(3)

# 'ok' function listeners

calls_0 = []


def function_listener_0():
    calls_0.append(True)


calls_1 = []


def function_listener_1(new):
    calls_1.append(new)


calls_2 = []


def function_listener_2(name, new):
    calls_2.append((name, new))


calls_3 = []


def function_listener_3(obj, name, new):
    calls_3.append((obj, name, new))


calls_4 = []


def function_listener_4(obj, name, old, new):
    calls_4.append((obj, name, old, new))


# 'fail' function listeners

exceptions_from = []


def failing_function_listener_0():
    exceptions_from.append(0)
    raise Exception('error')


def failing_function_listener_1(new):
    exceptions_from.append(1)
    raise Exception('error')


def failing_function_listener_2(name, new):
    exceptions_from.append(2)
    raise Exception('error')


def failing_function_listener_3(obj, name, new):
    exceptions_from.append(3)
    raise Exception('error')


def failing_function_listener_4(obj, name, old, new):
    exceptions_from.append(4)
    raise Exception('error')


class TestDynamicNotifiers(unittest.TestCase):

    #### 'TestCase' protocol ##################################################

    def setUp(self):
        self.exceptions = []
        trait_notifiers.push_exception_handler(self._handle_exception)

    def tearDown(self):
        trait_notifiers.pop_exception_handler()

    #### Private protocol #####################################################

    def _handle_exception(self, obj, name, old, new):
        self.exceptions.append((obj, name, old, new))

    #### Tests ################################################################

    def test_dynamic_notifiers_methods(self):
        obj = DynamicNotifiers(ok=2)
        obj.ok = 3

        self.assertEqual(len(obj.rebind_calls_0), 2)

        expected_1 = [2, 3]
        self.assertEqual(expected_1, obj.rebind_calls_1)

        expected_2 = [('ok', 2), ('ok', 3)]
        self.assertEqual(expected_2, obj.rebind_calls_2)

        expected_3 = [(obj, 'ok', 2), (obj, 'ok', 3)]
        self.assertEqual(expected_3, obj.rebind_calls_3)

        expected_4 = [(obj, 'ok', 0, 2), (obj, 'ok', 2, 3)]
        self.assertEqual(expected_4, obj.rebind_calls_4)

    def test_dynamic_notifiers_methods_failing(self):
        obj = DynamicNotifiers()
        obj.fail = 1

        _py2to3.assertCountEqual(self, [0, 1, 2, 3, 4], obj.exceptions_from)
        self.assertEqual([(obj, 'fail', 0, 1)]*5, self.exceptions)

    def test_dynamic_notifiers_functions(self):
        obj = DynamicNotifiers()

        obj.on_trait_change(function_listener_0, 'ok')
        obj.on_trait_change(function_listener_1, 'ok')
        obj.on_trait_change(function_listener_2, 'ok')
        obj.on_trait_change(function_listener_3, 'ok')
        obj.on_trait_change(function_listener_4, 'ok')

        obj.ok = 2
        obj.ok = 3

        expected_1 = [2, 3]
        self.assertEqual(expected_1, calls_1)

        expected_2 = [('ok', 2), ('ok', 3)]
        self.assertEqual(expected_2, calls_2)

        expected_3 = [(obj, 'ok', 2), (obj, 'ok', 3)]
        self.assertEqual(expected_3, calls_3)

        expected_4 = [(obj, 'ok', 0, 2), (obj, 'ok', 2, 3)]
        self.assertEqual(expected_4, calls_4)

    def test_priority_notifiers_first(self):

        obj = DynamicNotifiers()

        expected_high = set([1, 3])
        expected_low = set([0, 2])

        obj.on_trait_change(obj.low_priority_first, 'priority_test')
        obj.on_trait_change(obj.high_priority_first, 'priority_test',
                            priority=True)
        obj.on_trait_change(obj.low_priority_second, 'priority_test')
        obj.on_trait_change(obj.high_priority_second, 'priority_test',
                            priority=True)

        obj.priority_test = None

        high = set(obj.prioritized_notifications[:2])
        low = set(obj.prioritized_notifications[2:])

        self.assertSetEqual(expected_high, high)
        self.assertSetEqual(expected_low, low)


    def test_dynamic_notifiers_functions_failing(self):
        obj = DynamicNotifiers()

        obj.on_trait_change(failing_function_listener_0, 'fail')
        obj.on_trait_change(failing_function_listener_1, 'fail')
        obj.on_trait_change(failing_function_listener_2, 'fail')
        obj.on_trait_change(failing_function_listener_3, 'fail')
        obj.on_trait_change(failing_function_listener_4, 'fail')

        obj.fail = 1

        _py2to3.assertCountEqual(self, [0, 1, 2, 3, 4], obj.exceptions_from)
        # 10 failures: 5 are from the internal dynamic listeners, see
        # test_dynamic_notifiers_methods_failing
        self.assertEqual([(obj, 'fail', 0, 1)] * 10, self.exceptions)

    def test_object_can_be_garbage_collected(self):
        # Make sure that a trait object can be garbage collected even though
        # there are listener to its traits.

        import weakref

        obj = DynamicNotifiers()
        obj.on_trait_change(function_listener_0, 'ok')

        # Create a weak reference to `obj` with a callback that flags when the
        # object is finalized.
        obj_collected = []

        def obj_collected_callback(weakref):
            obj_collected.append(True)

        obj_weakref = weakref.ref(obj, obj_collected_callback)

        # Remove reference to `obj`, and check that the weak reference
        # callback has been called, indicating that it has been collected.
        del obj

        self.assertEqual(obj_collected, [True])

    def test_creating_notifiers_dont_create_cyclic_garbage(self):
        gc.collect()
        DynamicNotifiers()
        # When an object with dynamic listeners has no more references,
        # it should not create cyclic garbage.
        self.assertEqual(gc.collect(), 0)


if __name__ == '__main__':
    unittest.main()



# Standard library imports.
from traits.testing.unittest_tools import unittest

# Local imports.
from traits.util.camel_case import camel_case_to_python, camel_case_to_words


class CamelCaseTestCase(unittest.TestCase):

    def test_python_conversion(self):
        """ Does CamelCase -> Python name work?
        """
        c_names = [ 'GetFooBar', 'GetOBBTree', 'XMLDataReader',
                    'GetFooXML', 'HTMLIsSGML', '_SetMe', '_XYZTest',
                    'Actor2D', 'Actor3D', 'Actor6D', 'PLOT3DReader',
                    'Actor61Dimension', 'GL2PSExporter', 'Volume16Reader' ]
        t_names = [ 'get_foo_bar', 'get_obb_tree', 'xml_data_reader',
                    'get_foo_xml', 'html_is_sgml', '_set_me',
                    '_xyz_test', 'actor2d', 'actor3d', 'actor6_d',
                    'plot3d_reader', 'actor61_dimension',
                    'gl2ps_exporter', 'volume16_reader' ]
        for i, c_name in enumerate(c_names):
            t_name = camel_case_to_python(c_name)
            self.assertEqual(t_name, t_names[i])

    def test_word_conversion(self):
        """ Does CamelCase -> words work?
        """
        self.assertEqual(camel_case_to_words('FooBarBaz'), 'Foo Bar Baz')


if __name__ == "__main__":
    unittest.main()
